/* $Id: move.c,v 1.3 1999/06/08 00:41:10 marcus Exp $
******************************************************************************

   Graphics library for GGI. BSE render lib for 16 bit.

   Copyright (C) 1999 Andreas Beck	[becka@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <bse_int.h>

static void GGIbse_putimage(ggibse_t bob)
{
	if (bob->alpha) {
		int xx,yy;
		ggi_pixel *buf=bob->image;
		unsigned char *alpha=bob->alpha;
		for(yy=0;yy<bob->h;yy++)
			for(xx=0;xx<bob->w;xx++) {
				switch(*alpha) {
					case 0:	/* Opaque */
						ggiPutPixel(bob->vis,bob->x+xx,bob->y+yy,*buf);
						break;
					case 0xff:	/* Transparent */
						break;
					default:	/* Partially Transparent*/
					{
						ggi_color sprcol,bgcol;
						ggi_pixel bgpix;
						ggiGetPixel(bob->vis,bob->x+xx,bob->y+yy,&bgpix);
						ggiUnmapPixel(bob->vis,*buf ,&sprcol);
						ggiUnmapPixel(bob->vis,bgpix,&bgcol);
						sprcol.r=(sprcol.r*(255-*alpha)+bgcol.r**alpha)/255;
						sprcol.g=(sprcol.g*(255-*alpha)+bgcol.g**alpha)/255;
						sprcol.b=(sprcol.b*(255-*alpha)+bgcol.b**alpha)/255;
						ggiPutPixel(bob->vis,bob->x+xx,bob->y+yy,ggiMapColor(bob->vis,&sprcol));
					}
				}
				alpha++;
				buf++;
			}
	} else {
		ggiPutBox(bob->vis,bob->x,bob->y,bob->w,bob->h,bob->image);
	}
}

int GGIbse_stubs_es_move(ggibse_t bob,int x,int y)
{
	if (x!=bob->x||y!=bob->y) {
		ggiPutBox(bob->vis,bob->x,bob->y,bob->w,bob->h,bob->backgr);
		ggiGetBox(bob->vis,bob->x=x,bob->y=y,bob->w,bob->h,bob->backgr);
		GGIbse_putimage(bob);
	}
	return 0;
}

int GGIbse_stubs_es_show(ggibse_t bob,int show)
{
	if (show) {
		ggiGetBox(bob->vis,bob->x,bob->y,bob->w,bob->h,bob->backgr);
		GGIbse_putimage(bob);
	} else {
		ggiPutBox(bob->vis,bob->x,bob->y,bob->w,bob->h,bob->backgr);
	}
	return 0;
}

int GGIbse_stubs_bob_move(ggibse_t bob,int x,int y)
{
	bob->x=x;bob->y=y;
	GGIbse_putimage(bob);
	return 0;
}

int GGIbse_stubs_bob_show(ggibse_t bob,int show)
{
	if (show)
		GGIbse_putimage(bob);
	return 0;
}

int GGIbse_stubs_setimage(ggibse_t bob,ggi_visual_t vis,int x,int y,
			unsigned char *alpha)
{
	if (bob->image==NULL) bob->image=malloc(bob->w*bob->h*sizeof(ggi_pixel));
	if (bob->backgr==NULL) bob->backgr=malloc(bob->w*bob->h*sizeof(ggi_pixel));
	if (alpha && bob->alpha==NULL) bob->alpha=malloc(bob->w*bob->h*sizeof(unsigned char));
	if (alpha==NULL && bob->alpha) { free(bob->alpha);bob->alpha=NULL; }
	if (alpha) memcpy(bob->alpha,alpha,bob->w*bob->h*sizeof(unsigned char));
	ggiGetBox(bob->vis,0,0,bob->w,bob->h,bob->backgr);
	ggiCrossBlit(vis,x,y,bob->w,bob->h,bob->vis,0,0);
	if (bob->alpha) {
		int xx,yy;
		ggi_pixel *buf=bob->image;
		for(yy=0;yy<bob->h;yy++)
			for(xx=0;xx<bob->w;xx++) {
				ggiGetPixel(bob->vis,xx,yy,buf++);
			}
	} else {
		ggiGetBox(bob->vis,0,0,bob->w,bob->h,bob->image);
	}
	ggiPutBox(bob->vis,0,0,bob->w,bob->h,bob->backgr);
	return 0;
}

int GGIbse_stubs_alloc(ggibse_t bob)
{
	switch(bob->type) {
		case BSE_BOB:
			bob->move=GGIbse_stubs_bob_move;
			bob->show=GGIbse_stubs_bob_show;
			bob->setimage=GGIbse_stubs_setimage;
			return 0;
		case BSE_EMUSPRITE:
			bob->move=GGIbse_stubs_es_move;
			bob->show=GGIbse_stubs_es_show;
			bob->setimage=GGIbse_stubs_setimage;
			return 0;
		default:
			break;
	}
	return GGI_EARGINVAL;
}
